/*--------------------------------------------------------------------------------------+
|
|  $Source: MstnExamples/Annotations/ChangeCaseTool/ChangeCaseTool.cpp $
|
|  $Copyright: (c) 2016 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------+
| ChangeCaseTool application source                                                    |
+--------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------+
|                                                                                      |
| Demonstrates the use of Text API to copy and modify the properties of text in        |
| the Textblock e.g. changing the case of the text                                     |
| This tool is made generic, as in all tools which involve the logic of copy and modify|
| can make use of this pattern and add particular tool's application                   |
| logic to this pattern                                                                |
|                                                                                      |
| This tool changes the case of Text element.                                          |
| The user is prompted to select a Text element and is allowed to                      |
| select Text elements having strings containing only characters other than numbers.   |
| On valid selection a Textblock is extracted from the selected Text element           |
| and the case of the text in the Textblock is changed i.e.from lowercase              |
| to uppercase or from uppercase to lowercase                                          |
| The new Text element is created with the above changed Textblock and is              |
| placed at the new location                                                           |
+--------------------------------------------------------------------------------------*/


#pragma once
#include <Mstn/MdlApi/MdlApi.h>
#include <DgnView/AccuDraw.h>
#include <DgnView/DgnElementSetTool.h>
#include <DgnPlatform/ITextEdit.h>
#include <DgnPlatform/TextHandlers.h>

#include "ModifyTextTool.h"
#include "ModifyTextToolIds.h"
#include "ModifyTextToolCmd.h"
#include "ITextModify.h"

USING_NAMESPACE_BENTLEY_DGNPLATFORM;


/*=================================================================================**//**
* @bsiclass                                                               Bentley Systems
+===============+===============+===============+===============+===============+======*/
struct          ChangeCaseTool : public DgnElementSetTool
    {
    private:
        TextBlockPtr              m_textBlock;
        ModifyToolState           m_toolState;
        IModifyTextPtr            m_modifyText;

    protected: ChangeCaseTool	(int toolID, IModifyText* modifyText) :
    	DgnElementSetTool		(toolID),
    	m_toolState				(ModifyToolState::TOOLSTATE_Locating),
    	m_modifyText			(modifyText)
	{
  	}

    private: StatusInt _OnElementModify(EditElementHandleR eeh) override { return SUCCESS; } // pure virtual method, sub-class must override!
    private: bool _OnResetButton(DgnButtonEventCR ev) override { _OnRestartTool(); return true; }


/*----------------------------------------------------------------------------------**//**
* Returns true if the selection is a valid text element.
*
* @param[in]        hitPath            Hit path of the selected element.
*
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: bool  IsSelectionValid(HitPathCP hitPath)
	{
        if (!hitPath)
            return nullptr;

        TextBlockPtr textBlock = GetSelectedText (hitPath);
        if (textBlock.IsNull() || textBlock->IsEmpty())
            return false;

        // Check if the string in the Textblock is valid string or not.
        return m_modifyText->IsSelectionValid (*(textBlock.get ()));

	}
/*----------------------------------------------------------------------------------**//**
* Returns TextBlockPtr of the validated Text element.
*
* @param[in]        hitPath            Hit path of the selected element.
*
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: TextBlockPtr GetSelectedText(HitPathCP hitPath)
	{
        if (!hitPath)
            return nullptr;

        // Construct an element handle from the HitPath.
        ElementHandle eh(hitPath->GetHeadElem(), hitPath->GetRoot());

        // Check if the selection is a Text element.
        ITextQueryCP textQuery = eh.GetITextQuery();
        if (!textQuery || !textQuery->IsTextElement(eh))
            return nullptr;

        // Extract the TextBlock from the Text element.
        ITextPartIdPtr textPart;
        TextBlockPtr textBlock = textQuery->GetTextPart(eh, *textPart);
        if (textBlock.IsNull() || textBlock->IsEmpty())
            return nullptr;

        return textBlock;
	}
/*---------------------------------------------------------------------------------**//**
* Creates a Text element at a given pick pont.
*
* @param[out]       eehTextElem        EditElementHandle of the newly created text element .
* @param[in]        ev                 Button event which gives the pick point where the new element is placed.
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: StatusInt CreateTextElement(EditElementHandleR eehTextElem, DgnButtonEventCR ev)
	{
        if (m_textBlock.IsNull())
            return ERROR;

        m_textBlock->SetUserOrigin(*ev.GetPoint());

        return (TEXTBLOCK_TO_ELEMENT_RESULT_Success != (TextHandlerBase::CreateElement(eehTextElem, nullptr, *m_textBlock)) ? ERROR : SUCCESS);
	}


/*---------------------------------------------------------------------------------**//**
* Sets the state of the tool to either TOOLSTATE_Locating or TOOLSTATE_Dynamics.
* The user is prompted for actions based on the state the tool.
*
* @param[in]        toolState        Current state of the tool.
*
@bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: void SetToolState(ModifyToolState& toolState)
	{
        m_toolState = toolState;
	}


/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: void    _OnPostInstall() override
	{
        //Enables picking of elements
        _BeginPickElements();
        mdlAutoLocate_setComplexComponentMode(ComponentMode::Innermost); //NOTE: Must be called after BeginPickElements!

        // Set the tool state as'Locating' which will prompt the user to 'Select' elements.
        ModifyToolState toolState = ModifyToolState::TOOLSTATE_Locating;
        SetToolState(toolState);

        __super::_OnPostInstall();
	}
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: void _SetupAndPromptForNextAction() override
	{
        m_modifyText->SetupAndPromptForNextAction(m_toolState);
 	}
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: bool _OnPostLocate(HitPathCP path, WStringR cantAcceptReason) override
	{
        if (!DgnElementSetTool::_OnPostLocate(path, cantAcceptReason))
            return false;

        // Allow only text elements having strings containing alphabets to be selected.
        return IsSelectionValid(path);
	}
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: bool _OnDataButton(DgnButtonEventCR ev) override
	{
        if (m_textBlock.IsNull() || m_textBlock->IsEmpty() || m_modifyText.IsNull())
  		{
            //Locate an element and get path of it.
            HitPathCP path = _DoLocate(ev, true, ComponentMode::Innermost);
            if (!path)
                return false;

            //Gets the TextBlock from the selected Text element.
            m_textBlock = GetSelectedText(path);
            if (m_textBlock.IsNull() || m_textBlock->IsEmpty())
                return false;

            // Modifies the properties of the Text element.
            if (SUCCESS != m_modifyText->ModifyText(*m_textBlock))
                return false;

            ModifyToolState toolState = ModifyToolState::TOOLSTATE_Dynamics;
            SetToolState(toolState);

            _SetupAndPromptForNextAction();

            _BeginDynamics();
		}
        else
    	{
			// On 'Accept' action create a Text element and commit it to the file.
			EditElementHandle textElem;
			if ((SUCCESS == CreateTextElement(textElem, ev)) && textElem.IsValid())
			{
				//Add the newly created text element to the model.
				textElem.AddToModel();

				_OnRestartTool ();
			}
		}

        return true;
	}


/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: void _OnDynamicFrame(DgnButtonEventCR ev) override
	{
		// Create the text element and temporarily displays it.
		EditElementHandle textElem;
		if ((SUCCESS == CreateTextElement(textElem, ev)) && textElem.IsValid())
		{
			RedrawElems redrawElems;
			redrawElems.SetDrawMode(DRAW_MODE_TempDraw);
			redrawElems.SetDrawPurpose(DrawPurpose::Dynamics);
			redrawElems.SetViewport(ev.GetViewport());
			redrawElems.SetRedrawOp(this);
			redrawElems.DoRedraw(textElem);
		}
	}
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    private: void _OnRestartTool() override
	{
		InstallNewInstance(GetToolId());
	}

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
    public: static void InstallNewInstance(int toolId)
	{
        IModifyTextPtr modifyText = ModifyFactory::CreateTextModify(ModifyTextOperation::CHANGECASE);
        if (modifyText.IsNull())
            return;
        ChangeCaseTool* tool = new ChangeCaseTool(toolId, modifyText.get());
        tool->InstallTool();
	}

}; // ChangeCaseTool
/*---------------------------------------------------------------------------------**//**
* Function that is associated with the command number for starting the tool.
*
* @param[in]        unparsed        Additional input supplied after command string.
*
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+--*/
Public void ChangeCase_start(WCharCP /*unparsed*/)
{
	// NOTE: Call the method to create/install the tool, RefCounted classes don't have public constructors...
	ChangeCaseTool::InstallNewInstance(CMDNAME_ChangeCaseMdl);
}

