/*--------------------------------------------------------------------------------------+
|
|  $Source: MstnExamples/Annotations/ChangeTextTool/ChangeTextTool.cpp $
|
|  $Copyright: (c) 2020 LA Solutions Ltd. All rights reserved. $
|  $Copyright: (c) 2016 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------+
| ChangeTextTool application source                                                    |
+--------------------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------+
|                                                                                      |
| Demonstrates the use of Text API to cmodify the properties of text in        		   |
| the Textblock e.g. changing the text to a user-supplied string.                      |
| This tool is made generic, as in all tools which involve the logic of copy and modify|
| can make use of this pattern and add particular tool's application                   |
| logic to this pattern                                                                |
|                                                                                      |
| This tool changes Text element.                                                      |
| The user is prompted to select a Text element.and is allowed to                      |
| On valid selection a Textblock is extracted from the selected Text element           |
| and the text in the Textblock is changed i.e. to the user-supplied string.           |
| The new Text element is created with the above changed Textblock and is              |
| placed at the new location                                                           |
+--------------------------------------------------------------------------------------*/


#pragma once
#include <Mstn/MdlApi/MdlApi.h>
#include <DgnView/AccuDraw.h>
#include <DgnView/DgnElementSetTool.h>
#include <DgnPlatform/ITextEdit.h>
#include <DgnPlatform/TextHandlers.h>

#include "ModifyTextTool.h"
#include "ModifyTextToolIds.h"
#include "ModifyTextToolCmd.h"
#include "ITextModify.h"

USING_NAMESPACE_BENTLEY_DGNPLATFORM;

	///	<summary>ChangeTextTool class definition.</summary>
struct          ChangeTextTool : public DgnElementSetTool
{
    private:
        TextBlockPtr        m_textBlock;
        ModifyToolState     m_toolState;
        IModifyTextPtr      m_modifyText;
        WString				m_newText;

	///	<summary>ChangeTextTool constructor.</summary>
    protected: ChangeTextTool(int toolID, IModifyText* modifyText, WCharCP newText) :
    	DgnElementSetTool	(toolID),
    	m_toolState			(ModifyToolState::TOOLSTATE_Locating),
    	m_modifyText		(modifyText),
    	m_newText 			(newText)
	{
		WString s;
		s.Sprintf (L"ChangeTextTool (%s)", newText);
		NotificationManager::OutputMessage(NotifyMessageDetails(OutputMessagePriority::Debug, s.GetWCharCP(), s.GetWCharCP(), 0, OutputMessageAlert::None));
	}

    private: StatusInt _OnElementModify(EditElementHandleR eeh) override { return SUCCESS; } // pure virtual method, sub-class must override!
    private: bool _OnResetButton(DgnButtonEventCR ev) override { _OnRestartTool(); return true; }

	///	<summary>Is the selection a valid element for this tool?</summary>
	///	<returns>True if the selection is a valid text element.</returns>
    private: bool  IsSelectionValid(HitPathCP hitPath)
	{
		if (!hitPath)
			return false;

		TextBlockPtr textBlock = GetSelectedText (hitPath);
		if (textBlock.IsNull() || textBlock->IsEmpty())
			return false;
		return true;
	}
	///	<summary>Get and validate selected text.</summary>
	///	<param name="hitPath" >Hit path of the accepted element.</param>
	///	<returns>TextBlockPtr containing text from selected element.</returns>
    private: TextBlockPtr GetSelectedText(HitPathCP hitPath)
	{
		if (!hitPath)
			return nullptr;

		// Construct an element handle from the HitPath.
		ElementHandle eh(hitPath->GetHeadElem(), hitPath->GetRoot());

		// Check if the selection is a Text element.
		ITextQueryCP textQuery = eh.GetITextQuery();
		if (!textQuery || !textQuery->IsTextElement(eh))
			return nullptr;

		// Extract the TextBlock from the Text element.
		ITextPartIdPtr textPart;
		TextBlockPtr textBlock = textQuery->GetTextPart(eh, *textPart);
		if (textBlock.IsNull() || textBlock->IsEmpty())
			return nullptr;

		return textBlock;
	}
	///	<summary>Creates a Text element at a given pick point</summary>
	///	<param name="eehTextElem" >The new text element.</param>
	///	<param name="ev" >Button event which gives the pick point where the new element is placed.</param>
    private: StatusInt CreateTextElement(EditElementHandleR eehTextElem, DgnButtonEventCR ev)
	{
		if (m_textBlock.IsNull())
			return ERROR;

		CaretPtr	start	= m_textBlock->CreateStartCaret ();
		CaretPtr	end		= m_textBlock->CreateEndCaret ();
		m_textBlock->SetUserOrigin(*ev.GetPoint());
		m_textBlock->ReplaceText (m_newText.c_str (), *start, *end);

		return (TEXTBLOCK_TO_ELEMENT_RESULT_Success != (TextHandlerBase::CreateElement(eehTextElem, nullptr, *m_textBlock)) ? ERROR : SUCCESS);
	}
	///	<summary>Sets the state of the tool to either TOOLSTATE_Locating or TOOLSTATE_Dynamics.</summary>
	///	<remarks>The user is prompted for actions based on the state the tool.</remarks>
	///	<param name="toolState" >The new state of the tool.</param>
    private: void SetToolState(ModifyToolState& toolState)
	{
		m_toolState = toolState;
	}
	///	<summary>Prepare the tool for use.</summary>
    private: void    _OnPostInstall() override
	{
		//Enables picking of elements
		_BeginPickElements();
		mdlAutoLocate_setComplexComponentMode(ComponentMode::Innermost); //NOTE: Must be called after BeginPickElements!

		// Set the tool state as'Locating' which will prompt the user to 'Select' elements.
		ModifyToolState toolState = ModifyToolState::TOOLSTATE_Locating;
		SetToolState(toolState);

		__super::_OnPostInstall();
	}
	///	<summary>Prepare to restart the command and to pick more text.</summary>
    private: void _SetupAndPromptForNextAction() override
	{
		m_modifyText->SetupAndPromptForNextAction(m_toolState);
	}
	///	<summary>Filter the user's choice of element.</summary>
    private: bool _OnPostLocate(HitPathCP path, WStringR cantAcceptReason) override
	{
		if (!DgnElementSetTool::_OnPostLocate(path, cantAcceptReason))
			return false;

		// Allow only text elements having strings containing alphabets to be selected.
		return IsSelectionValid(path);
	}
	///	<summary>What to when user clicks datapoint to accept a text element.</summary>
    private: bool _OnDataButton(DgnButtonEventCR ev) override
	{
		if (m_textBlock.IsNull() || m_textBlock->IsEmpty() || m_modifyText.IsNull())
		{
			//Locate an element and get path of it.
			HitPathCP path = _DoLocate(ev, true, ComponentMode::Innermost);
			if (!path)
				return false;

			//Gets the TextBlock from the selected Text element.
			m_textBlock = GetSelectedText(path);
			if (m_textBlock.IsNull() || m_textBlock->IsEmpty())
				return false;

			// Modifies the properties of the Text element.
			if (SUCCESS != m_modifyText->ModifyText(*m_textBlock))
				return false;

			ModifyToolState toolState = ModifyToolState::TOOLSTATE_Dynamics;
			SetToolState(toolState);

			_SetupAndPromptForNextAction();

			_BeginDynamics();
		}
		else
		{
			// On 'Accept' action create a Text element and commit it to the file.
			EditElementHandle textElem;
			if ((SUCCESS == CreateTextElement(textElem, ev)) && textElem.IsValid())
			{
				//Add the newly created text element to the model.
				textElem.AddToModel();

				_OnRestartTool ();
			}
		}

		return true;
	}
	///	<summary>What to draw in dynamics, while the command is in progress.</summary>
    private: void _OnDynamicFrame(DgnButtonEventCR ev) override
	{
		// Create the text element and temporarily displays it.
		EditElementHandle textElem;
		if ((SUCCESS == CreateTextElement(textElem, ev)) && textElem.IsValid())
		{
		RedrawElems redrawElems;
		redrawElems.SetDrawMode(DRAW_MODE_TempDraw);
		redrawElems.SetDrawPurpose(DrawPurpose::Dynamics);
		redrawElems.SetViewport(ev.GetViewport());
		redrawElems.SetRedrawOp(this);
		redrawElems.DoRedraw(textElem);
		}
	}
	///	<summary>Restart this tool.</summary>
    private: void _OnRestartTool() override
	{
		InstallNewInstance(GetToolId(), m_newText.c_str ());
	}
	///	<summary>Static method to install a new instance of this command.</summary>
	public: static void InstallNewInstance(int toolId, WCharCP  newText)
	{
		IModifyTextPtr modifyText = ModifyFactory::CreateTextModify(ModifyTextOperation::CHANGETEXT);
		if (modifyText.IsNull())
			return;
		ChangeTextTool* tool = new ChangeTextTool(toolId, modifyText.get(), newText);
		tool->InstallTool();
	}
};

///	<summary>Function that is associated with the command number for starting the tool.</summary>
///	<param name="unparsed" >Replacement text.</param>
Public void ChangeText_start(WCharCP unparsed)
{
	if (WString::IsNullOrEmpty (unparsed))
	{
		// Issue message about empty string
		WString warn (L"Usage: CHANGETEXT <new text>");
		NotificationManager::OutputMessage(NotifyMessageDetails(OutputMessagePriority::Warning, warn.GetWCharCP(), warn.GetWCharCP(), 0, OutputMessageAlert::Balloon));

		return;
	}
    // NOTE: Call the method to create/install the tool, RefCounted classes don't have public constructors...
    ChangeTextTool::InstallNewInstance(CMDNAME_ChangeTextMdl, unparsed);
}
